use tl;
use reqwest;
use serde_json::{self, Value, json};
use std::{fs::{write, File}, io::Write};


#[tokio::main]
async fn main() {
    let url = "https://wiki.kmods.space/show/klib-res-timecrystal/itemDescriptor";
    let webdata = reqwest::get(url).await.expect("request failed").text().await.expect("request failed");


    let dom = tl::parse(webdata.as_str(), tl::ParserOptions::default()).expect("page is not html");
    let parser = dom.parser();
    let element = dom.get_element_by_id("__NUXT_DATA__").expect("__NUXT_DATA__ not found").get(parser).unwrap();
    let data = element.inner_text(parser);
    let mut json: Vec<Value> = serde_json::from_str(&data).unwrap();
    //println!("{:#?}", json);

    let mut v = json[0].clone();
    recurse(&mut v, &mut json);
    let result_b = v[1]["data"].as_object().unwrap().values().collect::<Vec<_>>();
    let result = result_b.get(0).unwrap();
    let mut file = File::create(format!("{}.json", &result["extraInformations"]["name"].as_str().unwrap())).unwrap();
    file.write(serde_json::to_string_pretty(&result).unwrap().as_bytes()).expect("failed to write json");

}

fn recurse<'a>(val: &'a mut Value, src: &'a Vec<Value>) {
    //println!("val before mutation: {}", val);
    match val {
        Value::Array(arr) => {
            for val in arr.iter_mut() {
                recurse(val, src);
            };
        }
        Value::Object(map) => {
            for (key, val) in map.iter_mut() {
                recurse(val, src);
            }
        }
        Value::Number(num) => {
            let i = num.as_u64().unwrap();
            let mut r = src.get(i as usize).unwrap().clone();
            match r  {
                Value::Number(_) => {}
                _ => { recurse(&mut r, src); }
            }
            *val = r;
        }
        _ => {}
    }
    //println!("val after mutation: {}", val);
}